/*=======================================================================================================*/
/*      Distributed Speech Recognition                                                                   */
/*      ETSI ES 202 050 Advanced Front-End Feature Extraction Algorithm & Compression Algorithm &        */
/*      ETSI ES 202 212 Extended Advanced Front-End Feature Extraction Algorithm & Compression Algorithm */  
/*      Bit exact C-language software implementation using ETSI basic operations library                 */
/*      3GPP TS 26.243 v 6.1.0 November 2004                                                                  */
/*      ANSI C code for the Fixed-Point Distributed Speech Recognition Extended Advanced Front-end       */
/*=======================================================================================================*/
/*---------------------------------------------------------------------------
*
* FILE NAME: ParmInterface.c
* PURPOSE:   This file contains...TO FILL IN...
*
*---------------------------------------------------------------------------*/
/*-----------------
* File Inclusions
*-----------------*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "typedef.h"
#include "basic_op.h"
#include "count.h"


#include "ParmInterface_B.h"
/*--------------------------------------------
* The followings include the module-specific 
* and exported methods (or functions).
*--------------------------------------------*/
#include "NoiseSupExports_B.h"
#include "WaveProcExports_B.h"
#include "CompCepsExports_B.h"
#include "PostProcExports_B.h"
#if defined(F_16KHZ)
#   include "16kHzProcExports_B.h"
#endif
#include "VADExports_F.h"
#if defined(XAFE)
#	include "ParmType.h"
#	include "rvc_pitch_B.h"
#endif
#ifndef SIZE_SCRATCH_ADVPROCESS
#	define	SIZE_SCRATCH_ADVPROCESS		482
#endif
static void CvFeatInt2Float(X_INT16 *FeatInt16, COEF_TYPE *FeatFloat) ;

/*----------------------------------------------------------------------------
 * FUNCTION NAME: AdvProcessAlloc
 *
 * PURPOSE:       Allocate memory for FEParamsX_F structure       
 *
 * INPUT:         
 *   SamplingFrequency     
 *
 * OUTPUT:
 *   
 *
 * RETURN VALUE:
 *   FEParmasX_F structure pointer
 *
 *---------------------------------------------------------------------------*/
FEParamsX_F* AdvProcessAlloc_B (int SamplingFrequency)
{
    FEParamsX_F *pFEParX = (FEParamsX_F *) calloc (1, sizeof (FEParamsX_F));
   
    pFEParX->DoNoiseSupAlloc = DoNoiseSupAlloc_B;
    pFEParX->DoNoiseSupInit = DoNoiseSupInit_B;
    pFEParX->DoNoiseSup = DoNoiseSup_B;
    pFEParX->DoNoiseSupDelete = DoNoiseSupDelete_B;

    pFEParX->DoWaveProcAlloc = DoWaveProcAlloc_B;
    pFEParX->DoWaveProcInit = DoWaveProcInit_B;
    pFEParX->DoWaveProc = DoWaveProc_B;
    pFEParX->DoWaveProcDelete = DoWaveProcDelete_B;

    pFEParX->DoCompCepsAlloc = DoCompCepsAlloc_B;
    pFEParX->DoCompCepsInit = DoCompCepsInit_B;
    pFEParX->DoCompCeps = DoCompCeps_B;
    pFEParX->DoCompCepsDelete = DoCompCepsDelete_B;
    pFEParX->DoPostProcAlloc = DoPostProcAlloc_B;
    pFEParX->DoPostProcInit = DoPostProcInit_B;
    pFEParX->DoPostProc = DoPostProc_B;
    pFEParX->DoPostProcDelete = DoPostProcDelete_B;
#if defined(F_16KHZ) 
#endif  
    pFEParX->DoVADProc = DoVADProc_F;
    pFEParX->DoVADDelete = DoVADDelete_F;
    pFEParX->DoVADInit   = DoVADInit_F;
    pFEParX->DoVADAlloc  = DoVADAlloc_F;
    pFEParX->DoVADFlush  = DoVADFlush_F;
    
    /*----------------------------------------------
    * allocate module-specific expansion structure
    *----------------------------------------------*/
    if (pFEParX->DoNoiseSupAlloc != NULL) pFEParX->NSX = pFEParX->DoNoiseSupAlloc ();
    if (pFEParX->DoWaveProcAlloc != NULL) pFEParX->WPX = pFEParX->DoWaveProcAlloc ();
    if (pFEParX->DoCompCepsAlloc != NULL) pFEParX->CCX = pFEParX->DoCompCepsAlloc ();
    if (pFEParX->DoPostProcAlloc != NULL) pFEParX->PPX = pFEParX->DoPostProcAlloc ();
    if (pFEParX->DoVADAlloc != NULL)  pFEParX->VADX = pFEParX->DoVADAlloc ();
    
    pFEParX->SamplingFrequency = SamplingFrequency;
#if defined(F_16KHZ)    
    if (SamplingFrequency == 16000)
    {
        pFEParX->SamplingFrequency = 8000;
        pFEParX->Do16kHzProc = 1;
        pFEParX->pData16k = (DataFor16kProc_F *) Do16kProcAlloc_B ();
    }
    else
        pFEParX->Do16kHzProc = 0;
#else
    if(SamplingFrequency == 16000) {
        fprintf(stderr,"\nABORT ERROR : 16Khz not allowed for this version\n");
        exit(0);
    }
#endif  
    return pFEParX;
}

/*----------------------------------------------------------------------------
 * FUNCTION NAME: AdvProcessInit
 *
 * PURPOSE:       Initialize AdvProcess data structure 
 *
 * INPUT:
 *   pFEParX      pointer to FEParamsX_F structure     
 *
 * OUTPUT:        Initialized FEParamsX_F structure
 *
 *
 * RETURN VALUE:  none
 *   
 *
 *---------------------------------------------------------------------------*/
void AdvProcessInit_B (FEParamsX_F *pFEParX)
{
	if (pFEParX->SamplingFrequency == SAMPLING_FREQ_1 * 1000)
    {
        pFEParX->FrameLength = FRAME_LENGTH;
        pFEParX->FrameShift = FRAME_SHIFT;
        pFEParX->FFTLength = FFT_LENGTH;
    }
    else if (pFEParX->SamplingFrequency == SAMPLING_FREQ_3 * 1000)
    {
        pFEParX->FrameLength = FRAME_LENGTH;
        pFEParX->FrameShift = FRAME_SHIFT;
        pFEParX->FFTLength = FFT_LENGTH;
    }
    else
    {
        fprintf (stderr, "ERROR:   Invalid sampling frequency '%d'!\r\n",
            pFEParX->SamplingFrequency);
        exit(0);
    }
    
    {
        int FrameShift = pFEParX->FrameShift;
#if defined(F_16KHZ)        
        if (pFEParX->Do16kHzProc == 1) FrameShift *= 2;
#endif      
    }
    if (pFEParX->DoNoiseSupInit != NULL) pFEParX->DoNoiseSupInit (pFEParX);
    if (pFEParX->DoWaveProcInit != NULL) pFEParX->DoWaveProcInit (pFEParX);
    if (pFEParX->DoCompCepsInit != NULL) pFEParX->DoCompCepsInit (pFEParX);
    if (pFEParX->DoPostProcInit != NULL) pFEParX->DoPostProcInit (pFEParX);
    if (pFEParX->DoVADInit != NULL) pFEParX->DoVADInit (pFEParX);
#if defined(F_16KHZ)
    if (pFEParX->Do16kHzProc == 1)
    {
        Do16kProcInit_B (pFEParX);
    }
#endif  
    pFEParX->denoisedBuf32 = 
        BufIn32Alloc (pFEParX->FrameLength +                         // at least FrameLength
        (pFEParX->FrameLength % pFEParX->FrameShift) + // integer number times FrameShift
        1);                                            // for pre-emphasis

    pFEParX->offsetDenoisedFrame = -pFEParX->FrameLength;
    pFEParX->FrameCounter = 0;
    pFEParX->NonZeroFrameOnset = 0;
    pFEParX->ZeroFrameCounter = 0;
    pFEParX->NbSamplesToRead = pFEParX->FrameShift;
    {
        int i ;
        for(i=0;i<MAX_NOISE_SHFT;i++) {
            pFEParX->tabShiftNoiseToCep[i] = 31 ;
        }
    }
#if defined(F_16KHZ)
    if (pFEParX->Do16kHzProc == 1) 
        pFEParX->NbSamplesToRead  = 2 * pFEParX->FrameShift;
#endif
}

/*----------------------------------------------------------------------------
* FUNCTION NAME: DoAdvProcess
*
* PURPOSE:       Processing for a frame 
*
* INPUT:
*   CurrentFrame Current Frame to process     
*   pFEParX      Pointer to front end parameter structure
* OUTPUT:
*   FeatureBuffer Buffer of features
*
* RETURN VALUE:
*   XTRUE        If a feature frame is output
*   FALSE        If no frame is output (due to delay)
*
*---------------------------------------------------------------------------*/


BOOLEAN DoAdvProcess_B (FILE_TYPE *CurrentFrameIn, 
                        COEF_TYPE *FeatureBuffer,
                        FEParamsX_F *pFEParX) 
{
    int i;
    int FrameShift;
    X_INT16 *CurrentFrame = CurrentFrameIn;
    X_INT16 iNoiseSupFlag;
    X_INT32 *ptDenoised32;

    X_INT16 scratchAdvProcess[SIZE_SCRATCH_ADVPROCESS];
    int FrameCheck ;
    long NonZeroFrameOnset;
    long ZeroFrameCounter;

#if defined(XAFE)
    X_INT16 iStftShift;
    X_INT16 decalSig;
    X_INT16 iProcessFlag;
#endif
    
    ZeroFrameCounter = pFEParX->ZeroFrameCounter;
    NonZeroFrameOnset = pFEParX->NonZeroFrameOnset;
    FrameShift  = pFEParX->FrameShift;                    move16();
#if defined(F_16KHZ)
    if (pFEParX->Do16kHzProc == 1) 
        FrameShift  = 2 * pFEParX->FrameShift;
#endif


        /*-----------------------------------------------------------------
        * get the pointeur where to input new Frameshift denoised samples
        * shift left data in denoisedBuf by FrameShift
        * and returns pointer to the new available part of
        * denoisedBuf => (denoisedBuf->size - FrameShift + 1)
    *-----------------------------------------------------------------*/
    ptDenoised32 = BufIn32ShiftToPut (pFEParX->denoisedBuf32, pFEParX->FrameShift);
    
    
    /*---------------------------------------------------
    * convert input samples from X_INT16 to SIGNAL_TYPE
    *---------------------------------------------------*/
    FrameCheck = 0;                                       move16();
    for (i=0 ; i<FrameShift; i++)
    {                                                       test();
        if(CurrentFrame[i] != 0) { FrameCheck = 1;        move16();}
    }
    
    test(); test();
    if ((FrameCheck != 0) || (NonZeroFrameOnset != 0))
    { 
        X_INT16 shiftNoiseToWav ;
        pFEParX->NonZeroFrameOnset = 1;                  move16();
        
#if defined(F_16KHZ)
        if (pFEParX->Do16kHzProc) {
        /*------------------------------------------------------------------------
        *  WARNING: 16kHz processing DM
        * curFrame [0 to (pFEParX->FrameShift - 1)] contains 0-4kHz data
        * curFrame [pFEParX->FrameShift to 2 * pFEParX->FrameShift - 1] contains
        * 4-8kHz data shifted to 0-4kHz freq. range
        *-------------------------------------------------------------------------*/
            Do16kProcessing_B (CurrentFrame, (DataFor16kProc_B *) (pFEParX->pData16k), FrameShift);
        }
#endif

#if defined(XAFE)
		{
				Shortword *pswStft = scratchAdvProcess ; /* size 258 */
				Shortword *pswSignal = scratchAdvProcess + 258; /* size 200 */

				iNoiseSupFlag = pFEParX->DoNoiseSup(CurrentFrame,ptDenoised32,pFEParX,&shiftNoiseToWav,pswStft,pswSignal,&iStftShift,&decalSig,&iProcessFlag);
				DoPitchExtract_B(pswStft,pswSignal,iStftShift,decalSig,iProcessFlag,pFEParX);
		}
#else
        iNoiseSupFlag = pFEParX->DoNoiseSup(CurrentFrame,ptDenoised32,pFEParX,&shiftNoiseToWav);
#endif

        test();
        if (iNoiseSupFlag)
        {
            { int i;
                for(i=MAX_NOISE_SHFT-1;i>0;i--) {
                    pFEParX->tabShiftNoiseToCep[i] = pFEParX->tabShiftNoiseToCep[i-1] ; move16();
                }
                pFEParX->tabShiftNoiseToCep[0] = shiftNoiseToWav ; move16();
            }
        /*---------------------------------------------------------------------------
        * the noise suppression module outputs "FrameShift" samples in denoisedBuf.
        * following modules will be processed only when denoisedBuf will be full.
            *---------------------------------------------------------------------------*/
#define MIN_NOISE_SHIFT 13  /* see NoiseSup_B.c 13 = SHFT_NOISE - 1 */
            test();
            if (pFEParX->offsetDenoisedFrame < 0) {
              pFEParX->offsetDenoisedFrame += pFEParX->FrameShift; add(0,0); move16();
            }
            test();
            if (pFEParX->offsetDenoisedFrame >= 0)
            {
                BOOLEAN rVal = FALSE;
                X_INT16 shiftWavToCep ; 
				X_INT16 *frameWavToCep  ; 
				X_INT32 *frameBuf32 = (X_INT32 *)scratchAdvProcess ; move16();
				frameWavToCep = (X_INT16 * ) frameBuf32; move16();
                BufIn32GetLast (pFEParX->denoisedBuf32, frameBuf32, pFEParX->FrameLength + pFEParX->offsetDenoisedFrame + 1); add(0,0); add(0,0); 
                {
                    int i ;
                    int iEnd;
                    X_INT16 shft, shftMin = 0 ; 
                    X_INT16 curShft;
                    X_INT32 iMax;
					
                    move16();
                    /* Determine max sample */
                    iMax= 0x80 ; move32();  /* to assure ( shftMin +  MIN_NOISE_SHIFT) < 35 )
					=> (0x4000 << 15) >> (35 - MIN_NOISE_SHIFT) */
					
                    curShft = sub(pFEParX->tabShiftNoiseToCep[0],MIN_NOISE_SHIFT); 
                    iEnd= add(add(pFEParX->FrameLength,pFEParX->offsetDenoisedFrame),1) ;
                    for(i=40;i<FRAME_SHIFT;i++) {
                        X_INT32 iabs = L_shr(L_abs(frameBuf32[iEnd-1-i]),curShft);
                        iMax = iMax | iabs ; logic32();
                    }
                    curShft = sub(pFEParX->tabShiftNoiseToCep[1],MIN_NOISE_SHIFT); 
                    for(i=FRAME_SHIFT;i<2*FRAME_SHIFT;i++) {
                        X_INT32 iabs = L_shr(L_abs(frameBuf32[iEnd-1-i]),curShft);
                        iMax = iMax | iabs ; logic32();
                    }
                    curShft = sub(pFEParX->tabShiftNoiseToCep[2],MIN_NOISE_SHIFT); 
                    for(i=2*FRAME_SHIFT;i<3*FRAME_SHIFT;i++) {
                        X_INT32 iabs = L_shr(L_abs(frameBuf32[iEnd-1-i]),curShft);
                        iMax = iMax | iabs ; logic32();
                    }
                    curShft = sub(pFEParX->tabShiftNoiseToCep[3],MIN_NOISE_SHIFT); 
                    for(i=3*FRAME_SHIFT;i<FRAME_BUF_SIZE;i++) {
                        X_INT32 iabs = L_shr(L_abs(frameBuf32[iEnd-1-i]),curShft);
                        iMax = iMax | iabs ; logic32();
                    }
                    /* Compute desired shift */
                    shftMin = add(norm_l(iMax),MIN_NOISE_SHIFT - 1) ;
                    test();
                    if(sub(shftMin,15) < 0 ) {  shftMin = 15 ; move16() ; }
                    /* Shift data accounting for current shift value on window basis */
                    shft = sub(pFEParX->tabShiftNoiseToCep[0],shftMin);
                    test();
                    if(shft > 0) {
                        for(i=40;i<FRAME_SHIFT;i++) {
                            frameBuf32[iEnd -1 - i  ] = L_shr_r(frameBuf32[iEnd-1-i] ,shft ) ; move32();
                        }
                    } else {
                        test();
                        if (shft < 0) {
                            for(i=40;i<FRAME_SHIFT;i++) {
                                frameBuf32[iEnd -1 - i  ] = L_shr(frameBuf32[iEnd-1-i] ,shft ) ; move32();
                            }
                        }
                    }
                    shft = sub(pFEParX->tabShiftNoiseToCep[1],shftMin); 
                    test();
                    if(shft > 0) {
                        for(i=FRAME_SHIFT;i<2*FRAME_SHIFT;i++) {
                            frameBuf32[iEnd -1 - i  ] = L_shr_r(frameBuf32[iEnd-1-i] ,shft ) ; move32();
                        }
                    } else {
                        test();
                        if (shft < 0) {
                            for(i=FRAME_SHIFT;i<2*FRAME_SHIFT;i++) {
                                frameBuf32[iEnd -1 - i  ] = L_shr(frameBuf32[iEnd-1-i] ,shft ) ; move32();
                            }
                        }
                    }
                    shft = sub(pFEParX->tabShiftNoiseToCep[2],shftMin); 
                    test();
                    if(shft > 0) {
                        for(i=2*FRAME_SHIFT;i<3*FRAME_SHIFT;i++) {
                            frameBuf32[iEnd -1 - i  ] = L_shr_r(frameBuf32[iEnd-1-i] ,shft ) ; move32();
                        }
                    } else {
                        test();
                        if (shft < 0) {
                            for(i=2*FRAME_SHIFT;i<3*FRAME_SHIFT;i++) {
                                frameBuf32[iEnd -1 - i  ] = L_shr(frameBuf32[iEnd-1-i] ,shft ) ; move32();
                            }
                        }
                    }
                    shft = sub(pFEParX->tabShiftNoiseToCep[3],shftMin); 
                    test();
                    if(shft > 0) {
                        for(i=3*FRAME_SHIFT;i<FRAME_BUF_SIZE;i++) {
                            frameBuf32[iEnd -1 - i  ] = L_shr_r(frameBuf32[iEnd-1-i] ,shft ) ; move32();
                        }
                    } else {
                        test();
                        if (shft < 0) {
                            for(i=3*FRAME_SHIFT;i<FRAME_BUF_SIZE;i++) {
                                frameBuf32[iEnd -1 - i  ] = L_shr(frameBuf32[iEnd-1-i] ,shft ) ; move32();
                            }
                        }
                    }
					
					/*---------------------
					* waveform processing
					*---------------------*/
                    if (pFEParX->DoWaveProc) rVal = pFEParX->DoWaveProc (frameBuf32+1,shftMin,frameWavToCep+1,&shiftWavToCep, pFEParX);
					
                }
                
                /*----------------------
                * cepstrum calculation
                *----------------------*/
                {
                    X_INT16 FeatInt[NUM_CEP_COEFF+1] ;
                    
                    if (pFEParX->DoCompCeps) {
                        rVal = pFEParX->DoCompCeps (frameWavToCep+1,shiftWavToCep, FeatInt, pFEParX);
                    }
  
					/*-----------------
                    * post processing
                    *-----------------*/
                    
                    if (pFEParX->DoPostProc) {
                        rVal = pFEParX->DoPostProc (FeatInt, pFEParX);
                    }
                    /*-----
                    * VAD
                    *-----*/
                    if (pFEParX->DoVADProc) {
                        rVal = pFEParX->DoVADProc (FeatInt, pFEParX);
                        CvFeatInt2Float(FeatInt,FeatureBuffer);
                        return rVal ;
						
                    } else {
                        return rVal;
                    }
                }
                
            }
            else
                return FALSE;
        }
        else
            return FALSE;
    }
    else
    {
        pFEParX->ZeroFrameCounter++; add(pFEParX->ZeroFrameCounter,1);move16();
        
        /*-------------------------
        * Create null MFCC vector
        *-------------------------*/
        for (i=0 ; i<=NUM_CEP_COEFF ; i++) { FeatureBuffer[i] = 0.0; move16();}
        
        /*---------------------
        * Update VAD variable
        *---------------------*/
        pFEParX->VAD = NON_SPEECH_FRAME; move16();

#if defined(XAFE)

      /*--------------------------------------------
       *  Output Unvoiced pitch & non-speech class
       *--------------------------------------------*/
      {
          float fPitchPeriod = 0.f;
          char cVoicingClass = '0';

          if (1!=fwrite(&fPitchPeriod,sizeof(float),1,pFEParX->pitch_fid))
          {
            fprintf (stderr, "ERROR: can't write pitch to file \r\n");
            exit(1);
          }

          if (1 != fprintf(pFEParX->class_fid,"%c",cVoicingClass))
          {
            fprintf (stderr, "ERROR: can't write voicing class to file \r\n");
            exit(1);
          }
      }

#endif
        
        return TRUE;
    }
}

/*----------------------------------------------------------------------------
* FUNCTION NAME: FlushAdvProcess
*
* PURPOSE:       Flushing of feature frames
*
* INPUT:
*
* OUTPUT:
*   FeatureBuffer Buffer of features
*
* RETURN VALUE:
*   XTRUE        If a feature frame is output
*   FALSE        If no frame is output (due to delay)
*
*---------------------------------------------------------------------------*/
BOOLEAN FlushAdvProcess_B (COEF_TYPE *FeatureBuffer, FEParamsX_F *pFEParX) 
{
    if (pFEParX->DoVADFlush) {
        X_INT16 FeatInt[NUM_CEP_COEFF+3] ;
        BOOLEAN rVal ;
        rVal = pFEParX->DoVADFlush (FeatInt, pFEParX);
       if (rVal)
		   CvFeatInt2Float(FeatInt,FeatureBuffer);

        return rVal ; 
    } else {
        return FALSE;
    }
}

/*----------------------------------------------------------------------------
* FUNCTION NAME: AdvProcessDelete
*
* PURPOSE:       
*
* INPUT:
*        
*
* OUTPUT:
*
*
* RETURN VALUE:
*   
*
*---------------------------------------------------------------------------*/
void AdvProcessDelete_B (FEParamsX_F **ppFEParX)
{    
    if ((*ppFEParX)->DoNoiseSupDelete != NULL) (*ppFEParX)->DoNoiseSupDelete ((*ppFEParX)->NSX);
    if ((*ppFEParX)->DoWaveProcDelete != NULL) (*ppFEParX)->DoWaveProcDelete ((*ppFEParX)->WPX);
    if ((*ppFEParX)->DoCompCepsDelete != NULL) (*ppFEParX)->DoCompCepsDelete ((*ppFEParX)->CCX);
    if ((*ppFEParX)->DoPostProcDelete != NULL) (*ppFEParX)->DoPostProcDelete ((*ppFEParX)->PPX);
    if ((*ppFEParX)->DoVADDelete != NULL) (*ppFEParX)->DoVADDelete ((*ppFEParX)->VADX);
    
    BufIn32Free ((*ppFEParX)->denoisedBuf32);
    
    /* 
       pFEParX->pData16k is not freed
       Something such as Do16kProcDelete_B(pFEParX->pData16k);
       would be needed
    */
    
    free (*ppFEParX);
    *ppFEParX = NULL;
}

/*----------------------------------------------------------------------------
* FUNCTION NAME: CvFeatInt2Float(X_INT16 *FeatInt16, COEF_TYPE *FeatFloat) 
*
* PURPOSE:       
*
* INPUT: FeatInt16
*        
*
* OUTPUT: FeatFloat
*
*
* RETURN VALUE:
*   
*
*---------------------------------------------------------------------------*/
static void CvFeatInt2Float(X_INT16 *FeatInt16, COEF_TYPE *FeatFloat) {
    int i ;
    for(i=0;i<NUM_CEP_COEFF-1;i++) {
        FeatFloat[i] = FeatInt16[i] / (float)PP_FACT_COEF  ;
    }
    FeatFloat[NUM_CEP_COEFF-1] = FeatInt16[NUM_CEP_COEFF-1] / (float)PP_FACT_C0  ;
    FeatFloat[NUM_CEP_COEFF] = FeatInt16[NUM_CEP_COEFF] / (float)PP_FACT_ENER  ;
    return ;

}

